// *****************************************************************
// ****                      - hard plAYer -                    ****
// ****              AY-38912 sound processor module            ****
// ****                  www.tolaemon.com/hplayer               ****
// ****                       Source code                       ****
// ****                       16-10-2008                        ****
// ****                     Jordi Bartolome                     ****
// ****                                                         ****
// **** IMPORTANT:                                              ****
// **** Using parts of this code means accepting all conditions ****
// **** exposed on the web: www.tolaemon.com/hplayer            ****
// *****************************************************************

/* 
&	AND	Bitwise AND
|	OR	Bitwise Inclusive OR
^	XOR	Bitwise Exclusive OR
~	NOT	Unary complement (bit inversion)
<<	SHL	Shift Left
>>	SHR	Shift Right
*/

#include "ayfwre.h"
#include "uart0.h"
#include "lcdg.h"
#include "timers.h"
#include "sdc.h"
#include "fat32.h"
#include "interface.h"
#include "ay3891x.h"


typedef struct{
	uint32_t byte_offset;//used to store some interesting byte offset positions inside the song,element 0 of the array, will always contain the position of the first frame (16 registers) after the file info data
	uint32_t number_of_frame;// is the number of the frame related to the byte_offset, so with this var we can check wich number of frame corresponds to the stored byte_offset
}t_song_position;

struct {
	uint8_t state; // controls the state of the Interface Screeen
	uint8_t song_name[MAX_AY_STRING]; // song name
	uint8_t song_author[MAX_AY_STRING]; // song author
	uint8_t song_comment[MAX_AY_STRING]; // song author
	uint32_t nbframes;// number of frames composing the song
	uint32_t read_frames;// number of frames read
	uint8_t clock_type;//AMSTRAD_CPC_CLOCK , SPECTRUM_ZX_CLOCK, ATARI_ST_CLOCK
	uint32_t loopframe;
	uint8_t registers[REGISTERS_PER_FRAME];
	t_song_position song_positions[MAX_BYTE_OFFSET_POSITIONS];	//used to store different song positions, element 0 of the array, will always 
																//contain the position of the first frame (16 registers) after the file info data,
																//we store the number of frame and the byte offset in the file where it starts
	uint16_t frame_timer_cicles;// used to control the playing speed, is the number of timer cicles between each registers update
} AY3891X_state;



/*	Description:
	Resets the AY3891X hardware audio generator, it means sets AY !Reset line to 0
	Recives:
	Returns:
 */
void AY3891X_Reset(){

	CLEAR_BDIR;
	CLEAR_BC1;

	SET_AY_RESET;
	TIMERS_BlockingPause(4000);
	CLEAR_AY_RESET;
	TIMERS_BlockingPause(8000);
	SET_AY_RESET;
	TIMERS_BlockingPause(4000);

};//AY3891X_Reset



/*	Description:
	Stores the current song's byte offset into the specified array position
	Recives:
		position with the position in the array where we want to store data in
	Returns:
		By value:
			1: if the song location could be saved
			-1: if was not possible to store the song location
 */
int8_t  AY3891X_SaveSongPosition(uint8_t position){
	int8_t return_value;

	if (position<MAX_BYTE_OFFSET_POSITIONS){
		
		FAT32_GetLastReadByteOffset(&(AY3891X_state.song_positions[position].byte_offset));
		AY3891X_state.song_positions[position].number_of_frame=AY3891X_state.read_frames;
		return_value=1;
		
	}else{

		return_value=-1;

	}//if
	
	return return_value;

};//AY3891X_SaveSongPosition



/*	Description:
	Loads the song byte offset stored into the specified array position
	Recives:
		position with the position in the array from where we want to load data
	Returns:
		By value:
			1: if the song location could be loaded
			-1: if was not possible to load the song location
 */
int8_t  AY3891X_LoadSongPosition(uint8_t position){
	int8_t return_value;

	// first af all check that the array index is valid and the array position is not empty
	if ((position<MAX_BYTE_OFFSET_POSITIONS)&&(AY3891X_state.song_positions[position].byte_offset!=0)){
		if (FAT32_SetReadByteOffset(AY3891X_state.song_positions[position].byte_offset)>=0){
			AY3891X_state.read_frames=AY3891X_state.song_positions[position].number_of_frame;
			// the specified offset has been set
			return_value=1;
		}else{
			// the stored offset could not be set
			return_value=-1;
		}//if
	}else{
		// the stored offset could not be set
		return_value=-1;
	}//if
	
	return return_value;

};//AY3891X_LoadSongPosition



/*	Description:
	Returns the number of the frame stored on the position specified by position 
	in the AY3891X_state.song_positions structure
	Recives:
		position the position of the song position array whose frame we want to read
	Returns:
		By refernce:
			frame: with the number of frame stored at the specfied position
		By value:
			1: if the specified song position exists and contains valid data
			-1: if was not possible to load the pecified song position
 */
int8_t  AY3891X_GetSongPositionFrame(uint8_t position,uint32_t * frame){
	int8_t return_value;

	// first af all check that the array index is valid and the array position is not empty
	if ((position<MAX_BYTE_OFFSET_POSITIONS)&&(AY3891X_state.song_positions[position].byte_offset!=0)){

			*frame=AY3891X_state.song_positions[position].number_of_frame;
			// the specified position frame was found
			return_value=1;

	}else{
		
		// the specified position does not exist or contains not valid data
		return_value=-1;

	}//if
	
	return return_value;

};//AY3891X_GetSongPositionFrame



/*	Description:
	Sets the value of the specified register
	Recives:
		By value:
			number: with the number of the register we want to modify
			value: with the value we want to set into the register
	Returns:
 */
void AY3891X_SetRegister(uint8_t number,uint8_t value){

	// first set address (BDIR=1, BC2=1, BC1=1 LATCH ADDRESS )
	SET_BDIR;
	SET_BC1;
	AY_DATA_ADDRESS=number;
			
	TIMERS_BlockingPause(5);
	
	// then set the value (BDIR=1, BC2=1, BC1=0 WRITE TO PSG )
	SET_BDIR;
	CLEAR_BC1;
	AY_DATA_ADDRESS=value;

	TIMERS_BlockingPause(5);

};//AY3891X_SetRegister



/*	Description:
	Resets and clears all the song info
	Recives:
	Returns:
 */
void AY3891X_ResetSong(){
	uint8_t i;
	
	AY3891X_state.song_name[0]='\0';
	AY3891X_state.song_author[0]='\0';
	AY3891X_state.song_comment[0]='\0';
	AY3891X_state.nbframes=0;
	AY3891X_state.read_frames=0;
	AY3891X_state.clock_type=AMSTRAD_CPC_CLOCK;
	AY3891X_state.loopframe=0;
	AY3891X_state.state=AY3891X_STATE_INACTIVE;
	AY3891X_state.frame_timer_cicles=FRAME_TIMER_CICLES;

	for (i=0;i<MAX_BYTE_OFFSET_POSITIONS;i++){
		AY3891X_state.song_positions[i].byte_offset=0;
		AY3891X_state.song_positions[i].number_of_frame=0;
	}//for

};//AY3891X_ResetSongInfo



/*	Description:
	Initializes the AY389X module, preparing all the microcontroller hardware ( SPI ) to work with the AY sound processor
	Recives:
	Returns:
 */
void AY3891X_Init(){

	DDRC=0xFF;// All pins are output for AY Address/Data
	DDRD=DDRD|0x80;// Pin 7 is configured as output for AY !Reset
	DDRG=DDRG|0x03;// Pins 0 1 are configured as output for BC1 (PG0) and BDIR (PG1)

	AY3891X_Reset();
	
	AY3891X_ResetSong();
	
};// AY3891X_Init



/*	Description:
	Reads each byte from the file comparing each read byte with the corresponding byte of the string
	Recives:
		string with the string we want to see if is stored in the file
	Returns:
		By value:
			 1: the received string matches with the file content
			-1: the received string does not match with the file content
			-2: an error has occured while reading the file
 */

int8_t AY3891X_CheckString(uint8_t * string ){
	uint8_t i;
	uint8_t read_byte;
	int8_t read_result;
	int8_t return_value=1;

	i=0;
	read_result=0;
	while((read_result>=0)&&(string[i]!='\0')&&(return_value==1)){

		read_result=FAT32_ReadByte(&read_byte);
		if (read_byte==string[i]){
			i++;
		}else if(read_result<0){
			return_value=-2;
		}else{
			return_value=-1;
		};//if
		
	};//while

	return return_value;

};//AY3891X_CheckString



/*	Description:
	Loads a string from the file and stores it's first max_bytes
	into the received string
	Recives:
		max_chars by value, with the maximum number of bytes that 
		can be stored into  the string, exceeding this value may 
		cause overwriting other vars in memory
	Returns:
		By reference:
			string containing the string read from the file
		By value:
			 1: the string has been read without problems
			<0: the file couldn't be read
 */
int8_t AY3891X_LoadString(uint8_t * string,uint8_t max_chars){
	uint8_t read_byte;
	int8_t read_result;
	int8_t i;

	i=0;
	do{
		read_result=FAT32_ReadByte(&read_byte);
		if ((read_result>=0)&&(i<max_chars-1)){
			string[i]=read_byte;
		}else if (i==(max_chars-1)){
			string[i]='\0';
		};//if
		i++;
	}while ( (read_result>=0)&&(read_byte!='\0')&&(i<255) );

	return read_result;

};//AY3891X_LoadString



/*	Description:
	Loads a Motorola Double Word from the file ( 32 bits: first
	byte the most significant, last one the less significant )
	The read pointershould be placed before rading the first 
	byte of the value
	Recives:
	Returns:
		By reference:
			
		By value:
			 1: the value has been read without problems
			<0: the file couldn't be read
 */

int8_t AY3891X_LoadMotorolaDoubleWord(uint32_t * double_word ){
	uint8_t read_byte;
	uint32_t aux_double_word;
	int8_t read_result;
	int8_t i;

	i=4;
	read_result=0;
	(*double_word)=0;
	while ((read_result>=0)&&(i>0)){
		read_result=FAT32_ReadByte(&read_byte);
		i--;
		aux_double_word=(uint32_t)read_byte;
		aux_double_word=aux_double_word<<(i*8);
		(*double_word)=(*double_word)|aux_double_word;
	};//while

	return read_result;

};//AY3891X_LoadMotorolaDoubleWord



/*	Description:
	Loads a Motorola Word from the file ( 16 bits: first
	byte the most significant, last one the less significant )
	The read pointershould be placed before rading the first 
	byte of the value
	Recives:
	Returns:
		By reference:
			
		By value:
			 1: the value has been read without problems
			<0: the file couldn't be read
 */

int8_t AY3891X_LoadMotorolaWord(uint16_t * motorola_word ){
	uint8_t read_byte;
	uint16_t aux_motorola_word;
	int8_t read_result;
	int8_t i;

	i=2;
	read_result=0;
	(*motorola_word)=0;
	while ((read_result>=0)&&(i>0)){
		read_result=FAT32_ReadByte(&read_byte);
		i--;
		aux_motorola_word=(uint16_t)read_byte;
		aux_motorola_word=aux_motorola_word<<(i*8);
		(*motorola_word)=(*motorola_word)|aux_motorola_word;
	};//while

	return read_result;

};//AY3891X_LoadMotorolaDoubleWord



/*	Description:
	Returns the name of the currently played song
	Recives:
		max_chars with the maximum allowed size of the
			returned string
	Returns:
		By reference:
			string with te song name
		By value:
 */
void AY3891X_YM_GetSongName(uint8_t * string, int8_t max_chars){
	int8_t i;
	
	i=0;
	while ((i<(max_chars-1))&&(i<(MAX_AY_STRING-1))&&(AY3891X_state.song_name[i]!='\0')){
		string[i]=AY3891X_state.song_name[i];
		i++;
	}//while
	
	string[i]='\0';

}//AY3891X_YM_GetSongName



/*	Description:
	Returns currently played song author's name
	Recives:
		max_chars with the maximum allowed size of the
			returned string
	Returns:
		By reference:
			string with the songs author
		By value:
 */
void AY3891X_YM_GetSongAuthor(uint8_t * string, int8_t max_chars){
	int8_t i;
	
	i=0;
	while ((i<(max_chars-1))&&(i<(MAX_AY_STRING-1))&&(AY3891X_state.song_author[i]!='\0')){
		string[i]=AY3891X_state.song_author[i];
		i++;
	}//while
	
	string[i]='\0';

}//AY3891X_YM_GetSongAuthor



/*	Description:
	Returns currently played song comments
	Recives:
		max_chars with the maximum allowed size of the
			returned string
	Returns:
		By reference:
			string with the song comments
		By value:
 */
void AY3891X_YM_GetSongComments(uint8_t * string, int8_t max_chars){
	int8_t i;
	
	i=0;
	while ((i<(max_chars-1))&&(i<(MAX_AY_STRING-1))&&(AY3891X_state.song_comment[i]!='\0')){
		string[i]=AY3891X_state.song_comment[i];
		i++;
	}//while
	
	string[i]='\0';

}//AY3891X_YM_GetSongComments



/*	Description:
	Returns the amount of frames of the current song and the number of frame
	which is being played
	Recives:
	Returns:
		By reference:
			nbframes the number of frames which compose the song
			read_frames the currently played frame number
		By value:
 */
void AY3891X_YM_GetTotalAndCurrentFrame(uint32_t * nbframes,uint32_t * read_frames){

	*nbframes=AY3891X_state.nbframes;
	*read_frames=AY3891X_state.read_frames;

}//AY3891X_YM_GetFrames



/*	Description:
	Sets the current played frame to the recived_frame, so places the read pointer 
	to the offset byte corresponding to that received_frame
	Recives:
		By value:
			current_frame with the number of frame we want to set as current frame
	Returns
		By value:
			TRUE if the new frame position has been set
			FALSE if the frame is not valid and has not been possible to set as current frame
*/
int8_t AY3891X_YM_SetCurrentFrame(uint32_t received_frame){
	uint32_t ui32_aux;

	// first we check that received frame is not beyond the limits of the currently played file
	if (received_frame<AY3891X_state.nbframes){
			
			// AY3891X_state.song_positions[0].byte_offset is always the offset of the firt frame, so 
			// we have to add to it the number of bytes corresponding to the received_frame
			ui32_aux=AY3891X_state.song_positions[0].byte_offset+received_frame*REGISTERS_PER_FRAME;
			
			if (FAT32_SetReadByteOffset(ui32_aux)>=0){
				AY3891X_state.read_frames=received_frame;
				
				return TRUE;
			}else{
				
				return TRUE;
			}
			
	}else{
		
		// received frame is beyond the limits of the file
		return FALSE;
		
	}//if

}//AY3891X_YM_SetCurrentFrame



/*	Description:
	Returns currently played song clock type
	Recives:
	Returns:
		By reference:
			the clock type
		By value:
 */
void AY3891X_YM_ClockType(uint8_t *clock_type){

	*clock_type=AY3891X_state.clock_type;

}//AY3891X_YM_ClockType



/*	Description:
	Returns currently played song loopframe
	Recives:
	Returns:
		By reference:
			the loop frame
		By value:
 */
void AY3891X_YM_LoopFrame(uint32_t *loop_frame){

	*loop_frame=AY3891X_state.loopframe;

}//AY3891X_YM_LoopFrame



/*	Description:
	Returns the state of the AY3891X motor, in orher words
	it tells if the song is being played or not
	Recives:
	Returns:
		By reference:
		By value:
			TRUE if the information is valid
			FALSE if the information is NOT valid
 */
uint8_t AY3891X_GetState(){

	return AY3891X_state.state;

}//AY3891X_YM_Ready



/*	Description:
	Allows modifiying externally the main motor state
	Recives:
		state with the state we want to set AY38912 main motor
	Returns:
 */
void AY3891X_SetState(uint8_t state){

	AY3891X_state.state=state;
	
}//AY3891X_SetState



/*	Description:
	Allows modifiying externally the song speed control var. This
	is the var which controlls the elapsed time betwen each load
    of a new frame ( group of 16 registers )
	Recives:
		timer_cicles with the new value
	Returns:
 */
void AY3891X_SetFrameTimerCicles(uint16_t timer_cicles){

	AY3891X_state.frame_timer_cicles=timer_cicles;
	
}//AY3891X_SetFrameTimerCicles



/*	Description:
	To get the value of the song speed control var. This is 
	the var which controlls the elapsed time betwen each load
    of a new frame ( group of 16 registers )
	
	Returns:
		By reference:
			timer_cicles with the new value
	Returns:
 */
void AY3891X_GetFrameTimerCicles(uint16_t * timer_cicles){

	*timer_cicles=AY3891X_state.frame_timer_cicles;
	
}//AY3891X_GetFrameTimerCicles



/*	Description:
	Loads YM file header general information and loads it into the LCD buffer
	Recives:
	Returns:
		By value:
			 1: the YM file header has been read correctly
			-1: it is not a valid YM file
 */
int8_t AY3891X_YM_LoadHeader(){
	uint32_t motorola_double_word=0;
	uint16_t motorola_word=0;
	uint8_t ui8_aux=0;
	
	// YM file header example:
	// "YM5!LeOnArD!" <= 12 bytes
	// 00 00 20 09 <= 4 bytes  nbFrame 8201
	// 00 00 00 01 <= 4 bytes Attrib 0   Attrib: A_STREAMINTERLEAVED = 1,A_DRUMSIGNED = 2,A_DRUM4BITS = 4,A_TIMECONTROL = 8,A_LOOPMODE = 16,
	// 00 00 <= 2 bytes nbDrum 0
	// 00 1E 84 80 <= 4 bytes 2000000 Clock
	// 00 32 <= 2 bytes 50 PlayerRate
	// 00 00 00 00 <= 4 bytes 0 loopFrame
	// 00 00 <= 2 bytes skip
	// "Dr Beat #1"'\0' <= n bytes Song Name
	// "Dr Beat"'\0' <= n bytes Song Autor
	// "Generated with SainT"'\0' <= n bytes Song Comment
	
	// checks if first bytes are YM
	if (AY3891X_CheckString("YM")<0){
		// first t4 bytes on the YM file should be YM6! , YM5!
		return -1;
	};//if

	// then checks the YM version ( YM5! or YM6! )
	FAT32_ReadByte(&ui8_aux);
	if ((ui8_aux!='5')&&(ui8_aux!='6')){
		return -1;
	};//if
	FAT32_ReadByte(&ui8_aux);// skips the '!' symbol
		
	// checks if next to the previous bytes apears the LeOnArD! string
	if (AY3891X_CheckString("LeOnArD!")<0){
		// next bytes should correspond to LeOnArD!
		return -1;
	};//if
	
	// reads the number of frames
	AY3891X_LoadMotorolaDoubleWord(&motorola_double_word);
	AY3891X_state.nbframes=motorola_double_word;

	// reads and checks the attributes
	AY3891X_LoadMotorolaDoubleWord(&motorola_double_word);
	if ((motorola_double_word&AY3891X_YM_INTERLEAVED)==1){
		return -1;
	};//if

	// reads digidrum effects
	AY3891X_LoadMotorolaWord(&motorola_word);
	if (motorola_word!=0){	
		return -1;
	};//if
		
	// reads the clock
	AY3891X_LoadMotorolaDoubleWord(&motorola_double_word);
	if (motorola_double_word==2000000){
		AY3891X_state.clock_type=ATARI_ST_CLOCK;
	}else if (motorola_double_word==1000000){
		AY3891X_state.clock_type=AMSTRAD_CPC_CLOCK;
	}else{
		AY3891X_state.clock_type=SPECTRUM_ZX_CLOCK;
	};//if

	// reads the sample rate
	AY3891X_LoadMotorolaWord(&motorola_word);
	if (motorola_word!=FRAMES_PER_SECOND){
		return -1;
	};//if

	// reads the loopFrame
	AY3891X_LoadMotorolaDoubleWord(&motorola_double_word);
	AY3891X_state.loopframe=motorola_double_word;
	
	// reads the skip bytes for extended options
	AY3891X_LoadMotorolaWord(&motorola_word);
	if (motorola_word!=0){
		return -1;
	};//if

	// song name
	if (AY3891X_LoadString(	AY3891X_state.song_name,MAX_AY_STRING)<0){
		return -1;
	};//if

	// song author
	if (AY3891X_LoadString(AY3891X_state.song_author,MAX_AY_STRING)<0){
		return -1;
	};//if

	// song commnets	
	if (AY3891X_LoadString(AY3891X_state.song_comment,MAX_AY_STRING)<0){
		return -1;
	};//if

	AY3891X_state.read_frames=0;
	
	AY3891X_state.frame_timer_cicles=FRAME_TIMER_CICLES;
	
	// stores the current file pointer offset (wich at this point should be
	// placed behind the header data) in the position 0 of the array
	AY3891X_SaveSongPosition(0);
	
	// the position 1 is used for storing the loop position, and it is initialized at the
	// first frame, but the user will be able to modify it later using the interface options
	AY3891X_SaveSongPosition(1);
	
	AY3891X_state.state=AY3891X_STATE_YM_READ_REGS_FROM_FILE;

	return 1;

};// AY3891X_YM_LoadHeader




/*	Description:
	Main state machine of the module which executes AY38912 main tasks
	Recives:
	Returns:
 */
void AY3891X_Motor(){
	static uint32_t frame_period_timer=0;
	uint8_t ui8_aux1;
	int8_t i8_aux1;
	uint8_t read_byte;

	switch(AY3891X_state.state){
		case AY3891X_STATE_INACTIVE:
			
			break;
			
		case AY3891X_STATE_YM_READ_REGS_FROM_FILE:
			
			// before reading we check if there are still frames to read
			if (AY3891X_state.read_frames>=AY3891X_state.nbframes){
				// all frames of the file have been read
				
				AY3891X_Reset();
				AY3891X_SetState(AY3891X_STATE_YM_STOPED);
				
			}else{
			
				// there are still frames to read so we read next 16 registers block
				i8_aux1=1;
				ui8_aux1=0;
				while ((i8_aux1>=0)&&(ui8_aux1<16)){
					i8_aux1=FAT32_ReadByte(&read_byte);
					AY3891X_state.registers[ui8_aux1]=read_byte;
					ui8_aux1++;
				};//while
				AY3891X_state.read_frames++;
				
				// if no error has occurred while reading from the file, on the 
				// next state machine cicle we will jump to AY3891X_STATE_YM_LOAD_REGS_TO_AY
				if (i8_aux1>=0){
					AY3891X_state.state=AY3891X_STATE_YM_LOAD_REGS_TO_AY;
				}else{
					AY3891X_state.state=AY3891X_STATE_YM_READ_ERROR;
				};//if
				
			};//if
			break;
			
		case AY3891X_STATE_YM_LOAD_REGS_TO_AY:
			
			if (TIMERS_Timer0ReachedTime(frame_period_timer+AY3891X_state.frame_timer_cicles)==1){
				
				ui8_aux1=0;
				while (ui8_aux1<16){
					AY3891X_SetRegister(ui8_aux1,AY3891X_state.registers[ui8_aux1]);
					ui8_aux1++;
				};//while
				AY3891X_state.state=AY3891X_STATE_YM_READ_REGS_FROM_FILE;
				frame_period_timer=TIMERS_Timer0GetTime();
				
			};//if
			
			break;
			
		case AY3891X_STATE_YM_STOPED:
			// don't do anything, wait until someboyd changes our state
			break;	

		case AY3891X_STATE_YM_PAUSED:
			// don't do anything, wait until someboyd changes our state
			break;	

		case AY3891X_STATE_YM_READ_ERROR:			
			// don't do anything, wait until someboyd changes our state
			break;
			
		default:
			break;
	};//switch

};//AY3891X_Motor



/*	Description:
	Sets a frecuency on the 3 channels 
	Recives:
		type: with the desired frecuency
	Returns:
 */
void AY3891X_SetFrecuency(int8_t type){

	AY3891X_Reset();

	switch (type){
		case 1:
			// set oscillator A frecuency
			AY3891X_SetRegister(R0,255);
			AY3891X_SetRegister(R1,15);
			// set oscillator B frecuency
			AY3891X_SetRegister(R2,255);
			AY3891X_SetRegister(R3,15);
			// set oscillator C frecuency
			AY3891X_SetRegister(R4,255);
			AY3891X_SetRegister(R5,15);
			
			// configure de MIXER
			AY3891X_SetRegister(R7,0xF8);
			
			break;
		case 2:
			// set oscillator A frecuency
			AY3891X_SetRegister(R0,255);
			AY3891X_SetRegister(R1,8);
			// set oscillator B frecuency
			AY3891X_SetRegister(R2,255);
			AY3891X_SetRegister(R3,8);
			// set oscillator C frecuency
			AY3891X_SetRegister(R4,255);
			AY3891X_SetRegister(R5,8);

			
			// configure de MIXER
			AY3891X_SetRegister(R7,0xF8);
			
			break;
		case 3:
			// set oscillator A frecuency
			AY3891X_SetRegister(R0,255);
			AY3891X_SetRegister(R1,4);
			// set oscillator B frecuency
			AY3891X_SetRegister(R2,255);
			AY3891X_SetRegister(R3,4);
			// set oscillator C frecuency
			AY3891X_SetRegister(R4,255);
			AY3891X_SetRegister(R5,4);
			
			// configure de MIXER
			AY3891X_SetRegister(R7,0xF8);
			
			// configure de MIXER
			AY3891X_SetRegister(R7,0xF8);
			
			break;
		case 4:
			// set oscillator A frecuency
			AY3891X_SetRegister(R0,255);
			AY3891X_SetRegister(R1,2);
			// set oscillator B frecuency
			AY3891X_SetRegister(R2,255);
			AY3891X_SetRegister(R3,2);
			// set oscillator C frecuency
			AY3891X_SetRegister(R4,255);
			AY3891X_SetRegister(R5,2);
			
			// configure de MIXER
			AY3891X_SetRegister(R7,0xF8);
			
			break;
		case 5:
			// set oscillator A frecuency
			AY3891X_SetRegister(R0,255);
			AY3891X_SetRegister(R1,1);
			// set oscillator B frecuency
			AY3891X_SetRegister(R2,255);
			AY3891X_SetRegister(R3,2);
			// set oscillator C frecuency
			AY3891X_SetRegister(R4,255);
			AY3891X_SetRegister(R5,1);
			
			// configure de MIXER
			AY3891X_SetRegister(R7,0xF8);
			
			break;
		case 6:
			// set oscillator A frecuency
			AY3891X_SetRegister(R0,255);
			AY3891X_SetRegister(R1,0);
			// set oscillator B frecuency
			AY3891X_SetRegister(R2,255);
			AY3891X_SetRegister(R3,0);
			// set oscillator C frecuency
			AY3891X_SetRegister(R4,255);
			AY3891X_SetRegister(R5,0);
			
			// configure de MIXER
			AY3891X_SetRegister(R7,0xF8);
			
			break;
		case 7:
			// set oscillator A frecuency
			AY3891X_SetRegister(R0,128);
			AY3891X_SetRegister(R1,0);
			// set oscillator B frecuency
			AY3891X_SetRegister(R2,128);
			AY3891X_SetRegister(R3,0);
			// set oscillator C frecuency
			AY3891X_SetRegister(R4,128);
			AY3891X_SetRegister(R5,0);
			
			// configure de MIXER
			AY3891X_SetRegister(R7,0xF8);
			
			break;
		case 8:
			// set oscillator A frecuency
			AY3891X_SetRegister(R0,64);
			AY3891X_SetRegister(R1,0);
			// set oscillator B frecuency
			AY3891X_SetRegister(R2,64);
			AY3891X_SetRegister(R3,0);
			// set oscillator C frecuency
			AY3891X_SetRegister(R4,64);
			AY3891X_SetRegister(R5,0);
			
			// configure de MIXER
			AY3891X_SetRegister(R7,0xF8);
			
			break;
		case 9:
			// set oscillator A frecuency
			AY3891X_SetRegister(R0,32);
			AY3891X_SetRegister(R1,0);
			// set oscillator B frecuency
			AY3891X_SetRegister(R2,32);
			AY3891X_SetRegister(R3,0);
			// set oscillator C frecuency
			AY3891X_SetRegister(R4,32);
			AY3891X_SetRegister(R5,0);
			
			// configure de MIXER
			AY3891X_SetRegister(R7,0xF8);
			
			break;
		case 'a':
			// set oscillator A frecuency
			AY3891X_SetRegister(R0,16);
			AY3891X_SetRegister(R1,0);
			// set oscillator B frecuency
			AY3891X_SetRegister(R2,16);
			AY3891X_SetRegister(R3,0);
			// set oscillator C frecuency
			AY3891X_SetRegister(R4,16);
			AY3891X_SetRegister(R5,0);
			
			// configure de MIXER
			AY3891X_SetRegister(R7,0xF8);
			
			break;
		case 'b':
		
			// configure de MIXER
			AY3891X_SetRegister(R7,0xC7);
			
			break;
		default:
			break;
	}//switch
			
			
	// configure oscillator A amplitude
	AY3891X_SetRegister(R10,0x08);
	// configure oscillator B amplitude
	AY3891X_SetRegister(R11,0x08);
	// configure oscillator C amplitude
	AY3891X_SetRegister(R12,0x08);


}//AY3891X_SetFrecuency



/*	Description:
	Tests routines, which lets check if the AY38912 works properly or not
	Recives:
		step: with the test routine to execute
	Returns:
	  By value:
	  By reference:
 */
void AY3891X_Test(int8_t step){
	uint8_t ui8_aux;
	uint8_t ui8_aux2;

	switch(step){
		case 1:
			SET_BDIR;
			SET_BC1;
			UART0_PrintString("Addr mode\r\n");
			UART0_ForceBufferTransmission();
			break;
		case 2:
			SET_BDIR;
			CLEAR_BC1;
			UART0_PrintString("Data mode\r\n");
			UART0_ForceBufferTransmission();
			break;
		case 3:
			SET_AY_RESET;
			UART0_PrintString("Set reset\r\n");
			UART0_ForceBufferTransmission();
			break;
		case 4:
			CLEAR_AY_RESET;
			UART0_PrintString("Clear reset\r\n");
			UART0_ForceBufferTransmission();
			break;
		case 5:
			PORTC=0xAA;
			UART0_PrintString("Set to AA\r\n");
			UART0_ForceBufferTransmission();
			break;
		case 6:
			ui8_aux=0;
			ui8_aux2=0;

			UART0_PrintString("Scan address\r\n");
			UART0_ForceBufferTransmission();
	
			for (ui8_aux=0;ui8_aux<255;ui8_aux++){
				
				AY3891X_SetRegister(ui8_aux,0XAA);
				
				// Sets the address we want to read
				DDRC=0xFF;// All pins are outputs
				SET_BDIR;
				SET_BC1;
				AY_DATA_ADDRESS=ui8_aux;
				TIMERS_BlockingPause(2000);
				
				// Reads the content of that address
				DDRC=0x00;// All pins are inputs for AY Address/Data
				CLEAR_BDIR;
				SET_BC1;
				UART0_PrintString("\r\nR");
				UART0_PrintUint(ui8_aux);
				UART0_PrintString(":");
				ui8_aux2=PINC;
				UART0_PrintUint(ui8_aux2);
				UART0_ForceBufferTransmission();
				TIMERS_BlockingPause(2000);
			};//for
			
			DDRC=0xFF;// All pins are output for AY Address/Data
			CLEAR_BDIR;
			CLEAR_BC1;
			break;
		case 7:
			ui8_aux=0;
			ui8_aux2=0;
			
			for (ui8_aux=0;ui8_aux<=15;ui8_aux++){
				// Sets the address we want to read
				DDRC=0xFF;// All pins are outputs
				SET_BDIR;
				SET_BC1;
				AY_DATA_ADDRESS=ui8_aux;
				TIMERS_BlockingPause(2000);
				
				// Reads the content of that address
				DDRC=0x00;// All pins are inputs for AY Address/Data
				CLEAR_BDIR;
				SET_BC1;
				UART0_PrintString("\r\nR");
				UART0_PrintUint(ui8_aux);
				UART0_PrintString(":");
				ui8_aux2=PINC;
				UART0_PrintUint(ui8_aux2);
				UART0_ForceBufferTransmission();
				TIMERS_BlockingPause(2000);
			};//for
			DDRC=0xFF;// All pins are output for AY Address/Data
			CLEAR_BDIR;
			CLEAR_BC1;
			break;
		case 8:
			UART0_PrintString("Trying 2\r\n");
			UART0_ForceBufferTransmission();
			
			AY3891X_Reset();

			UART0_PrintString("Reset done\r\n");
			UART0_ForceBufferTransmission();

			// set oscillator A frecuency
			AY3891X_SetRegister(R0,71);
			AY3891X_SetRegister(R1,0);
			// set oscillator B frecuency
			AY3891X_SetRegister(R2,142);
			AY3891X_SetRegister(R3,0);
			// set oscillator C frecuency
			AY3891X_SetRegister(R4,29);
			AY3891X_SetRegister(R5,1);

			// set noise frecuency
			AY3891X_SetRegister(R6,1);

			UART0_PrintString("Reset freq\r\n");
			UART0_ForceBufferTransmission();

			// configure de MIXER
			AY3891X_SetRegister(R7,0xE0);
			
			UART0_PrintString("Set mixer\r\n");

			// configure oscillator A amplitude
			AY3891X_SetRegister(R10,0x08);
			// configure oscillator B amplitude
			AY3891X_SetRegister(R11,0x08);
			// configure oscillator C amplitude
			AY3891X_SetRegister(R12,0x08);

			// set envelope fine period
			AY3891X_SetRegister(R13,0);
			// set envelope coarse period
			AY3891X_SetRegister(R14,0);
			// set envelope shape
			AY3891X_SetRegister(R15,0x00);
	
			break;
		case 9:
			UART0_PrintString("Set freq1\r\n");
			UART0_ForceBufferTransmission();
			AY3891X_SetFrecuency(1);
			break;
		case 'a':
			UART0_PrintString("Set freq2\r\n");
			AY3891X_SetFrecuency(2);
			break;
		case 'b':
			UART0_PrintString("Set freq3\r\n");
			AY3891X_SetFrecuency(3);
			break;
		case 'c':
			UART0_PrintString("Set freq4\r\n");
			AY3891X_SetFrecuency(4);
			break;	
		case 'd':
			UART0_PrintString("Set freq5\r\n");
			UART0_ForceBufferTransmission();
			AY3891X_SetFrecuency(5);
			break;
		case 'e':
			UART0_PrintString("Set freq6\r\n");
			AY3891X_SetFrecuency(6);
			break;
		case 'f':
			UART0_PrintString("Set freq7\r\n");
			AY3891X_SetFrecuency(7);
			break;
		case 'g':
			UART0_PrintString("Set freq8\r\n");
			AY3891X_SetFrecuency(8);
			break;
		case 'h':
			UART0_PrintString("Set freq9\r\n");
			AY3891X_SetFrecuency(9);
			break;
		case 'i':
			UART0_PrintString("Set freq a\r\n");
			AY3891X_SetFrecuency('a');
			break;
		case 'j':
			UART0_PrintString("Set freq b\r\n");
			AY3891X_SetFrecuency('b');
			break;
		default:
			break;
	};//switch

};//AY3891X_Test

