// *****************************************************************
// ****                      - hard plAYer -                    ****
// ****              AY-38912 sound processor module            ****
// ****                  www.tolaemon.com/hplayer               ****
// ****                       Source code                       ****
// ****                       16-10-2008                        ****
// ****                     Jordi Bartolome                     ****
// ****                                                         ****
// **** IMPORTANT:                                              ****
// **** Using parts of this code means accepting all conditions ****
// **** exposed on the web: www.tolaemon.com/hplayer            ****
// *****************************************************************
#ifndef _AY3891X_
#define _AY3891X_

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include "iocompat.h"

#define R0 0
#define R1 1
#define R2 2
#define R3 3
#define R4 4
#define R5 5
#define R6 6
#define R7 7
#define R10 8
#define R11 9
#define R12 10
#define R13 11
#define R14 12
#define R15 13

#define SET_BDIR PORTG=PORTG|0x02
#define CLEAR_BDIR PORTG=PORTG&0xFD
#define SET_BC1 PORTG=PORTG|0x01
#define CLEAR_BC1 PORTG=PORTG&0xFE
#define SET_AY_RESET PORTD=PORTD|0x80;
#define CLEAR_AY_RESET PORTD=PORTD&0x7F;
#define AY_DATA_ADDRESS PORTC


#define AMSTRAD_CPC_CLOCK 0
#define SPECTRUM_ZX_CLOCK 1
#define ATARI_ST_CLOCK 2

#define AY3891X_STATE_INACTIVE 0
#define AY3891X_STATE_YM_READ_REGS_FROM_FILE 2
#define AY3891X_STATE_YM_LOAD_REGS_TO_AY 3
#define AY3891X_STATE_YM_READ_ERROR 4
#define AY3891X_STATE_YM_STOPED 5
#define AY3891X_STATE_YM_PAUSED 6

#define MAX_BYTE_OFFSET_POSITIONS 2


#define AY3891X_YM_INTERLEAVED 1

#define MAX_AY_STRING 32

#define REGISTERS_PER_FRAME 16
#define FRAMES_PER_MINUTE ( 60 * FRAMES_PER_SECOND )
#define FRAMES_FORWARD_INTERVAL ( FRAMES_PER_SECOND*5);
#define FRAMES_REWIND_INTERVAL ( FRAMES_PER_SECOND*5);

#define FRAMES_PER_SECOND 50 // is the default FRAMES_PER_SECOND frame refresh frecuency
#define MILISECONDS_PER_FRAME 20// is the number of miliseconds that passes between each frame update
#define FRAME_TIMER_CICLES 3250 // are the cicles needed to load each frame each 0.02sec,( the default FRAMES_PER_SECOND frame refresh frecuency )
#define MIN_FRAME_TIMER_CICLES 250 // the minimium acepted value for FRAME_TIMER_CICLES
#define MAX_FRAME_TIMER_CICLES 64000 // the maximum acepted value for FRAME_TIMER_CICLES
#define FRAME_TIMER_CICLES_STEP 100// is the amount that the "speed counter" is increased or decreased when the user presses the up or down key


void AY3891X_Init();
void AY3891X_Test(int8_t step);
void AY3891X_StartPlaying();
void AY3891X_Motor();
int8_t AY3891X_YM_LoadHeader();
void AY3891X_YM_GetSongName(uint8_t * string, int8_t max_chars);
void AY3891X_YM_GetSongAuthor(uint8_t * string, int8_t max_chars);
void AY3891X_YM_GetSongComments(uint8_t * string, int8_t max_chars);
void AY3891X_YM_GetTotalAndCurrentFrame(uint32_t * nbframes,uint32_t * read_frames);
int8_t AY3891X_YM_SetCurrentFrame(uint32_t current_frame);
void AY3891X_YM_ClockType(uint8_t *clock_type);
void AY3891X_YM_LoopFrame(uint32_t *loop_frame);
void AY3891X_Reset();
void AY3891X_ResetSong();
uint8_t AY3891X_GetState();
void AY3891X_SetState(uint8_t state);
void AY3891X_GetFrameTimerCicles(uint16_t * timer_cicles);
void AY3891X_SetFrameTimerCicles(uint16_t timer_cicles);
int8_t  AY3891X_SaveSongPosition(uint8_t position);
int8_t  AY3891X_LoadSongPosition(uint8_t position);
int8_t  AY3891X_GetSongPositionFrame(uint8_t position,uint32_t * frame);

#endif

