// *****************************************************************
// ****                      - hard plAYer -                    ****
// ****     User interface, menu options and file explorer      ****
// ****                  www.tolaemon.com/hplayer               ****
// ****                       Source code                       ****
// ****                       16-10-2008                        ****
// ****                     Jordi Bartolome                     ****
// ****                                                         ****
// **** IMPORTANT:                                              ****
// **** Using parts of this code means accepting all conditions ****
// **** exposed on the web: www.tolaemon.com/hplayer            ****
// *****************************************************************
   
#include "interface.h"



struct {
	uint8_t state; // controls the state of the Interface Screeen
	uint8_t lcd_content_modified; // is the flag wich controls if the LCD has to be updated
	uint16_t file_expl_first;// is the N of the entry of the dir wich apears first on the LCD list
	uint16_t file_expl_selected;// wich of the N of entries of the list is currently selected
	uint16_t file_expl_numb_entries;// is the N of entries in the directory
	uint8_t sw_buttons_samples[MAX_KEYB_SAMPLES];//stores the state of the sw buttons after N reads, the first position stores the content of first read, the second position the content of second read ...
	uint8_t last_sw_button_sample;//is the number of samples stored into the sw_buttons_samples samples buffer
	uint8_t refresh_path_entries;// when this flag is true the LCD buffer is updated with the next NUMB_FILE_EXPL_ENTRIES,
					// this happens when the cursor reaches the top or the bottom of the LCD, then LCD char buffer is updated, this
					// is done to avoid read the memory card each time the cursor is moved, so entries are "stored" in LCD buffer
} INTERFACE_state;

uint8_t short_entry[MAX_NAME_SHORT_ENTRY+1+1];
uint8_t long_entry[MAX_LONG_NAME+1+1];
uint8_t file_info_string[MAX_AY_STRING];



/*	Description:
	Initializes the file explorer vars
	Recives: 
	Returns:
	 By value:
	 By reference:
 */
void INTERFACE_FileExplorerInit(){

	INTERFACE_state.file_expl_first=1;
	INTERFACE_state.file_expl_selected=1;//wich of the N elements of the list is selected
	INTERFACE_state.file_expl_numb_entries=0;
	INTERFACE_state.refresh_path_entries=TRUE;

};//INTERFACE_FileExplorerInit	



/*	Description:
	Initializes the interface main vars
	Recives: 
	Returns:
	 By value:
	 By reference:
 */
void INTERFACE_Init(){
	uint8_t ui8_aux;

	INTERFACE_state.state=INTERFACE_STATE_INTRO;
	INTERFACE_state.lcd_content_modified=TRUE;
	INTERFACE_FileExplorerInit();

	// resets the buffer were the keyboard samples are stored
	INTERFACE_state.last_sw_button_sample=0;
	for(ui8_aux=0;ui8_aux<MAX_KEYB_SAMPLES;ui8_aux++){
		INTERFACE_state.sw_buttons_samples[ui8_aux]=0;
	}//for	
	
};//INTERFACE_Init



/*	Description:
	Resets the buttons state to avoid reading them twice
	Recives: 
	Returns:
 */
void INTERFACE_ResetSwButtonsState(){
	uint8_t ui8_aux;
	
	// if any key is pressed we reset the sw_buttons_samples buffer
	if (INTERFACE_state.sw_buttons_samples[MAX_KEYB_SAMPLES-1]>0){

		// resets the buffer where the keyboard samples are stored
		INTERFACE_state.last_sw_button_sample=0;
		for(ui8_aux=0;ui8_aux<MAX_KEYB_SAMPLES;ui8_aux++){
			INTERFACE_state.sw_buttons_samples[ui8_aux]=0;
		}//for

	}//if
	
};//INTERFACE_ResetSwButtonsState



/*	Description:
	Reads the state of the sw_buttons and stores it into the INTERFACE_state.sw_buttons_samples buffer.
	Each time this functions is called checks the keyboard, and if detects that the user is pressing a key
	and it is the same one that the preoviously stored it is also stored, until the buffer is full. When 
	the buffer is full and all the samples are  identical to the previous one it means that the key state
    is stable and valid. Then, the keyboard polling is disabled during TIME_BOUNCING_FILTER and the keys 
	value is mantained in the budder untill the TIME_BOUNCING_FILTER ends. So if the keyboard is chekced 
	during this period with INTERFACE_CheckSwButton funciton it will return the value stored in the 
	INTERFACE_state.sw_buttons_samples buffer.
	
	|              [0][1][2][3]... N<--Number of sw button samples needed to process the key
	|------------+                 +XXXXXXXXXXXXXXXXXXXXXXXXX+
	|			 |				   |						 |
	|			 | PRESSING        |<--TIME_BOUNCING_FILTER->| During this time keys will be disabled
	|			 |		     KEY   |						 |
	+------------+-----------------+-------------------------+--------

	Recives: 
	Returns:
 */

void INTERFACE_LoadSwButtonsState(){
	static uint32_t bouncing_filter_timer=0;
	uint8_t ui8_aux;



	// applies a mask to get the PINF bits which correspond to the sw_buttons. As they work with inverted logic 
	// (0 pressed 1 not pressed) we have to invert them in order to have the right logic: 1 pressed 0 not pressed
	ui8_aux=(uint8_t)(~(PINF|INTERFACE_BUTTON_MASK));

	// if a key has been pressed and the sw_buttons are not dissabled for debouncing
	if ( (ui8_aux!=0) && (TIMERS_Timer0ReachedTime(bouncing_filter_timer+TIME_BOUNCING_FILTER)==1)){

		// checks if all the samples have been taken ( if the buffer is full) and if the sampled content
		// is the same that the one sampled in the previous cicle
		if ((INTERFACE_state.last_sw_button_sample<(MAX_KEYB_SAMPLES-1))&&
			(ui8_aux==INTERFACE_state.sw_buttons_samples[INTERFACE_state.last_sw_button_sample])){
				
				INTERFACE_state.last_sw_button_sample++;
				INTERFACE_state.sw_buttons_samples[INTERFACE_state.last_sw_button_sample]=ui8_aux;
			
				// if all the samples are equal and the buffer is full the timer is updated to disable polling during TIME_BOUNCING_FILTER cicles
				if (INTERFACE_state.last_sw_button_sample==MAX_KEYB_SAMPLES-1){
					bouncing_filter_timer=TIMERS_Timer0GetTime();
				}//if
				
		}else{
				
				INTERFACE_state.sw_buttons_samples[0]=ui8_aux;
				INTERFACE_state.last_sw_button_sample=0;
					
		}//if	

	
	};//if



};//INTERFACE_LoadSwButtonsState




/*	Description:
	Returns the heyboard state stored into the the INTERFACE_state.sw_buttons_samples buffer, it returns
	the last element stored in the buffer because if the pressed key samaples buffer has been filled it
	means that it has been possible to detect and sample the key and it is valid.
	Recives: 
	 By value:
		sw_button: with the button we want to check
	Returns:
	 By value:
		TRUE: if the button is pressed
		FALSE: if the button is not pressed
 */
uint8_t INTERFACE_CheckSwButton(uint8_t sw_buttons ){
	uint8_t ret_value;
 
	// we check if recived sw buttons are pressed or not
	if ((INTERFACE_state.sw_buttons_samples[MAX_KEYB_SAMPLES-1]&sw_buttons)>0){
		ret_value=TRUE;
	}else{
		ret_value=FALSE;
	};//if
	
	return ret_value;
	
};//INTERFACE_CheckSwButton



/*	Description:
	Interface's keyboard state machine
	Recives:
	Returns:
	 By value:
	 By reference:
 */
void INTERFACE_Keyboard_Motor(){
	static uint32_t timer_force_refersh_LCD=0;
	uint8_t ui8_aux1;
	int8_t i8_aux1;
	int8_t i8_aux2;
	uint16_t ui16_aux1;
	uint32_t ui32_aux1;
	uint32_t ui32_aux2;
	uint32_t ui32_aux3;
	
	INTERFACE_LoadSwButtonsState();

	// check the keys and other vars to get the current user interface state		
	switch (INTERFACE_state.state){
		
		case INTERFACE_STATE_INTRO:

			if (INTERFACE_CheckSwButton(SWITCH3|SWITCH4)){
			
					// shows a LCD message while checking de SD/MMC card
					LCDG_SetString(0,7,"                  ");
					LCDG_SetString(0,6,"                  ");
					LCDG_SetString(0,5,"                  ");
					LCDG_SetString(0,4,"   Reading card   ");
					LCDG_SetString(0,3,"                  ");
					LCDG_SetString(0,2,"                  ");
					LCDG_SetString(0,1,"                  ");
					LCDG_SetString(0,0,"                  ");
					LCDG_InvertRow(0xFF,0);// all lines unmarked
					LCDG_ForceLCDRefresh();
					TIMERS_NBlockingPauses(2,64000);

					// first we try to initialize the card and check if the card is ready
					FAT32_Reset();// reset the FAT32 structure
					i8_aux1=FAT32_Initialize();// loads the FAT32 basic info
					if (i8_aux1<0){
						// if the card is not ready we jump into the CARD ERROR STATE
						INTERFACE_state.state=INTERFACE_STATE_CARD_ERROR;
						INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
					}else{
						// if the card is ready we jump into the FILE_EXPLORER_STATE but first we initialize most important vars
						INTERFACE_FileExplorerInit();
						FAT32_GetNumberDirElements(&INTERFACE_state.file_expl_numb_entries);
						INTERFACE_state.state=INTERFACE_STATE_FILE_EXPLORER;
						INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
					};//if

			};//if
			break;
		
		case INTERFACE_STATE_FILE_EXPLORER:

			if (INTERFACE_CheckSwButton(SWITCH1)==TRUE){
			
				// if the selection cursor is not at the last entry yet, the cursor is increased to the next position
				if (INTERFACE_state.file_expl_selected<INTERFACE_state.file_expl_numb_entries){
					INTERFACE_state.file_expl_selected++;
					if (INTERFACE_state.file_expl_selected>=(INTERFACE_state.file_expl_first+NUMB_FILE_EXPL_ENTRIES)){
						INTERFACE_state.file_expl_first++;
						INTERFACE_state.refresh_path_entries=TRUE;// the entries will be added to the char buffer
					};//if
					INTERFACE_state.state=INTERFACE_STATE_FILE_EXPLORER;
					INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
				};//if
			};//if
			
			if (INTERFACE_CheckSwButton(SWITCH2)==TRUE){

				// if the selection cursor is not at the first entry yet, the cursor is decreased to the previous cursor
				if (INTERFACE_state.file_expl_selected>1){
					INTERFACE_state.file_expl_selected--;
					if (INTERFACE_state.file_expl_selected<INTERFACE_state.file_expl_first){
						INTERFACE_state.file_expl_first--;
						INTERFACE_state.refresh_path_entries=TRUE;// the entries will be added to the char buffer
					};//if
					INTERFACE_state.state=INTERFACE_STATE_FILE_EXPLORER;
					INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
				};//if
			};//if
			
			if (INTERFACE_CheckSwButton(SWITCH3)==TRUE){

				// if the cursor is placed in a directory or a file and user presses enter we try to load it
				ui16_aux1=INTERFACE_state.file_expl_selected;
				i8_aux2=FAT32_GetDirElementN(ui16_aux1,short_entry,&ui8_aux1,&ui32_aux1,&ui32_aux2,long_entry);
				if ((i8_aux2>=0)&&(FAT32_LoadDirElement(ui32_aux1,ui8_aux1,short_entry, ui32_aux2)>=0)){
					//  depending on the type of the entry ( file or directory ) one action or other is executed
					switch (ui8_aux1){	
							case 0:
							case 1:
								// the entry is a directory, we remain into the FILE_EXPLORER_STATE but first we initialize the FAT most important vars
								FAT32_GetNumberDirElements(&INTERFACE_state.file_expl_numb_entries);
								INTERFACE_state.file_expl_selected=1;
								INTERFACE_state.file_expl_first=1;
								INTERFACE_state.state=INTERFACE_STATE_FILE_EXPLORER;
								INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
								INTERFACE_state.refresh_path_entries=TRUE;// updates the entries in the LCD
								break;
							case 2:
								// the entry is a file, jump to the PLAYING FILE state
								INTERFACE_state.file_expl_selected=1;
								INTERFACE_state.file_expl_first=1;
								INTERFACE_state.state=INTERFACE_STATE_FILE_LOADING;
								INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
								break;
							default:
								// jump to the FILE ERROR state
								INTERFACE_state.state=INTERFACE_STATE_FILE_ERROR;
								INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
								break;
					};//switch						
				}else{
					// jump to the FILE ERROR state
					INTERFACE_state.state=INTERFACE_STATE_FILE_ERROR;
					INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
				};//if
			};//if
			
			if (INTERFACE_CheckSwButton(SWITCH4)==TRUE){
				INTERFACE_state.state=INTERFACE_STATE_INTRO;
				INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
			};//if
			break;
						
		case INTERFACE_STATE_FILE_LOADING:
			
			// if the cursor is placed in a directory or a file and user presses enter we try to load it
			ui16_aux1=INTERFACE_state.file_expl_selected;
			i8_aux2=FAT32_GetDirElementN(ui16_aux1,short_entry,&ui8_aux1,&ui32_aux1,&ui32_aux2,long_entry);
			if ((short_entry[9]=='Y')&&(short_entry[10]=='M')){
				
				// tries to read the YMfile header
				if (AY3891X_YM_LoadHeader()>=0){	
					INTERFACE_state.state=INTERFACE_STATE_FILE_PLAYING_SHOW_INFO;
					INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
				}else{
					INTERFACE_state.state=INTERFACE_STATE_FILE_ERROR;
				};//if
					
			}else{
				
				// File is not an YM file!
				INTERFACE_state.state=INTERFACE_STATE_FILE_ERROR;
					
			};//if
			break;
			
		case INTERFACE_STATE_FILE_PLAYING_SHOW_INFO:
		
			// checks if the file is being played
			if (AY3891X_GetState()==AY3891X_STATE_YM_READ_ERROR){
				AY3891X_Reset();// resets the audio IC
				AY3891X_ResetSong();// clears the song information
				INTERFACE_state.lcd_content_modified=TRUE;
				INTERFACE_state.state=INTERFACE_STATE_FILE_ERROR;
			};//if
						
			// is not necessary to update periodically the LCD
			
			// checks if PLAY - PAUSE button has been pressed
			if (INTERFACE_CheckSwButton(SWITCH1)==TRUE){
				
				ui8_aux1=AY3891X_GetState();
				if ((ui8_aux1==AY3891X_STATE_YM_READ_REGS_FROM_FILE)||(ui8_aux1==AY3891X_STATE_YM_LOAD_REGS_TO_AY)){
					// if the file is being played we set the AY3891X main motor in PAUSE state
					AY3891X_Reset();
					AY3891X_SetState(AY3891X_STATE_YM_PAUSED);
				}else if (ui8_aux1==AY3891X_STATE_YM_PAUSED){
					// if the file is PAUSED we set the AY3891X main motor in playing state
					AY3891X_SetState(AY3891X_STATE_YM_READ_REGS_FROM_FILE);
				}else if (ui8_aux1==AY3891X_STATE_YM_STOPED){
					// if the file is STOPPED we go back to the first position and set the AY3891X main motor in playing state
					AY3891X_LoadSongPosition(0);
					AY3891X_SetState(AY3891X_STATE_YM_READ_REGS_FROM_FILE);
				}//if
				
				INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
				
			};//if
			
			// if STOP is pressed the AY3891X state machine is placed into the STOPPED state
			if (INTERFACE_CheckSwButton(SWITCH2)==TRUE){
				AY3891X_Reset();
				AY3891X_SetState(AY3891X_STATE_YM_STOPED);
				INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
			};//if
			
			// if Enter is pressed the state machin jumps to the next PLAYING screen
			if (INTERFACE_CheckSwButton(SWITCH3)==TRUE){
				INTERFACE_state.state=INTERFACE_STATE_FILE_PLAYING_SHOW_TIME;
				INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
			};//if
			
			// if the user presses exit the song stops playing
			if (INTERFACE_CheckSwButton(SWITCH4)==TRUE){
				
				AY3891X_Reset();// resets the audio IC
				AY3891X_ResetSong();// clears the song information
				
				// restores the FAT32 structure with the pervious directory
				if (FAT32_RestorePreviousDirElement()>=0){
					// jump to the FILE EXPLORER state
					INTERFACE_state.state=INTERFACE_STATE_FILE_EXPLORER;
					INTERFACE_state.refresh_path_entries=TRUE;// updates the entries in the LCD
					INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
				}else{
					// jump to the FILE ERROR state
					INTERFACE_state.state=INTERFACE_STATE_FILE_ERROR;
					INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
				};//if
				
			};//if
			break;
			
		case INTERFACE_STATE_FILE_PLAYING_SHOW_TIME:
			
			// checks if the file is being played
			if (AY3891X_GetState()==AY3891X_STATE_YM_READ_ERROR){
				AY3891X_Reset();// resets the audio IC
				AY3891X_ResetSong();// clears the song information
				INTERFACE_state.lcd_content_modified=TRUE;
				INTERFACE_state.state=INTERFACE_STATE_FILE_ERROR;
			};//if
						
			// cheks if is time to refresh the LCD content
			if (TIMERS_Timer0ReachedTime(timer_force_refersh_LCD+TIME_FORCE_REFRESH_LCD)==1){
				INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
				timer_force_refersh_LCD=TIMERS_Timer0GetTime();
			};//if
			
			// if the REWIND buttons is pressed we go back some frames
			if (INTERFACE_CheckSwButton(SWITCH1)==TRUE){
				
				ui8_aux1=AY3891X_GetState();
				// only if the file is beying played, REWIND button is attended
				if ((ui8_aux1==AY3891X_STATE_YM_READ_REGS_FROM_FILE)||(ui8_aux1==AY3891X_STATE_YM_LOAD_REGS_TO_AY)){
					
					AY3891X_YM_GetTotalAndCurrentFrame(&ui32_aux1,&ui32_aux2);
					
					// checks if we have placed the frame beyond the limits of song
					ui32_aux3=FRAMES_REWIND_INTERVAL;
					if (ui32_aux2>ui32_aux3){
						ui32_aux2=ui32_aux2-FRAMES_REWIND_INTERVAL;
					}else{
						ui32_aux2=0;
					}//if
					AY3891X_YM_SetCurrentFrame(ui32_aux2);
					INTERFACE_state.lcd_content_modified=TRUE;
				
				};//if
				
			};//if
			
			// if the FOWARD button is pressed we go ahead some frames
			if (INTERFACE_CheckSwButton(SWITCH2)==TRUE){
				
				ui8_aux1=AY3891X_GetState();
				// only if the file is beying played REWIND button is attended
				if ((ui8_aux1==AY3891X_STATE_YM_READ_REGS_FROM_FILE)||(ui8_aux1==AY3891X_STATE_YM_LOAD_REGS_TO_AY)){
					
					AY3891X_YM_GetTotalAndCurrentFrame(&ui32_aux1,&ui32_aux2);
					
					// checks if we have placed the frame beyond the limits of song
					ui32_aux3=ui32_aux2+FRAMES_FORWARD_INTERVAL;
					if (ui32_aux3<ui32_aux1){
						ui32_aux2=ui32_aux2+FRAMES_FORWARD_INTERVAL;
					}else{
						ui32_aux2=ui32_aux1-1;
					}//if
					AY3891X_YM_SetCurrentFrame(ui32_aux2);	
					INTERFACE_state.lcd_content_modified=TRUE;	
					
				};//if
				
			};//if
			
			// if Enter is pressed the state machin jumps to the next PLAYING screen
			if (INTERFACE_CheckSwButton(SWITCH3)==TRUE){
				INTERFACE_state.state=INTERFACE_STATE_FILE_PLAYING_CHANGE_SPPED;
				INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
			};//if
			
			// if the exit buttons is pressed the song si stoped and the interface goes back to the file explorer
			if (INTERFACE_CheckSwButton(SWITCH4)==TRUE){
				
				AY3891X_Reset();// resets the audio IC
				AY3891X_ResetSong();// clears the song information
				
				// restores the FAT32 structure with the pervious directory
				if (FAT32_RestorePreviousDirElement()>=0){
					// jump to the FILE EXPLORER state
					INTERFACE_state.state=INTERFACE_STATE_FILE_EXPLORER;
					INTERFACE_state.refresh_path_entries=TRUE;// updates the entries in the LCD
					INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
				}else{
					// jump to the FILE ERROR state
					INTERFACE_state.state=INTERFACE_STATE_FILE_ERROR;
					INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
				};//if
				
			};//if
			
			break;
			
		case INTERFACE_STATE_FILE_PLAYING_CHANGE_SPPED:	
			
			// checks if the file is being played
			if (AY3891X_GetState()==AY3891X_STATE_YM_READ_ERROR){
				AY3891X_Reset();// resets the audio IC
				AY3891X_ResetSong();// clears the song information
				INTERFACE_state.lcd_content_modified=TRUE;
				INTERFACE_state.state=INTERFACE_STATE_FILE_ERROR;
			};//if
						
			// cheks if is time to refresh the LCD content
			if (TIMERS_Timer0ReachedTime(timer_force_refersh_LCD+TIME_FORCE_REFRESH_LCD)==1){
				INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
				timer_force_refersh_LCD=TIMERS_Timer0GetTime();// updates the timer
			};//if
				
			// Down arrow is pressed
			if (INTERFACE_CheckSwButton(SWITCH1)==TRUE){
				
				AY3891X_GetFrameTimerCicles(&ui16_aux1);		
				
				// checks that ui16_aux1 is not going to lowerlimit value
				if (ui16_aux1-FRAME_TIMER_CICLES_STEP>MIN_FRAME_TIMER_CICLES){
					ui16_aux1=ui16_aux1-FRAME_TIMER_CICLES_STEP;// decreases the lapse of time between each frame load
					AY3891X_SetFrameTimerCicles(ui16_aux1);
					INTERFACE_state.lcd_content_modified=TRUE;
				}//if
				
			};//if

			// Up arrow is pressed
			if (INTERFACE_CheckSwButton(SWITCH2)==TRUE){
				
				AY3891X_GetFrameTimerCicles(&ui16_aux1);
				
				// checks that ui16_aux1 is not going to cross the upper limit value
				if (ui16_aux1+FRAME_TIMER_CICLES_STEP<MAX_FRAME_TIMER_CICLES){
					ui16_aux1=ui16_aux1+FRAME_TIMER_CICLES_STEP;// increases the lapse of time between each frame load
					AY3891X_SetFrameTimerCicles(ui16_aux1);
					INTERFACE_state.lcd_content_modified=TRUE;
				};//if
				
			};//if
			
			// if Enter is pressed the state machin jumps to the next PLAYING screen
			if (INTERFACE_CheckSwButton(SWITCH3)==TRUE){
				INTERFACE_state.state=INTERFACE_STATE_FILE_PLAYING_LOOPING;
				INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
			};//if
			
			if (INTERFACE_CheckSwButton(SWITCH4)==TRUE){
				
				AY3891X_Reset();// resets the audio IC
				AY3891X_ResetSong();// clears the song information
				
				// restores the FAT32 structure with the pervious directory
				if (FAT32_RestorePreviousDirElement()>=0){
					// jump to the FILE EXPLORER state
					INTERFACE_state.state=INTERFACE_STATE_FILE_EXPLORER;
					INTERFACE_state.refresh_path_entries=TRUE;// updates the entries in the LCD
					INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
				}else{
					// jump to the FILE ERROR state
					INTERFACE_state.state=INTERFACE_STATE_FILE_ERROR;
					INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
				};//if
			};//if
			
			break;	
			
		case INTERFACE_STATE_FILE_PLAYING_LOOPING:	
			
			// checks if the file is being played
			if (AY3891X_GetState()==AY3891X_STATE_YM_READ_ERROR){
				AY3891X_Reset();// resets the audio IC
				AY3891X_ResetSong();// clears the song information
				INTERFACE_state.lcd_content_modified=TRUE;
				INTERFACE_state.state=INTERFACE_STATE_FILE_ERROR;
			};//if
						
			// cheks if is time to refresh the LCD content
			if (TIMERS_Timer0ReachedTime(timer_force_refersh_LCD+TIME_FORCE_REFRESH_LCD)==1){
				INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
				timer_force_refersh_LCD=TIMERS_Timer0GetTime();
			};//if
			
			// 's' buttons is pressed
			if (INTERFACE_CheckSwButton(SWITCH1)==TRUE){
				AY3891X_SaveSongPosition(1);
				INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
			};//if
			
			// 'l' buttons is pressed
			if (INTERFACE_CheckSwButton(SWITCH2)==TRUE){
			
				AY3891X_LoadSongPosition(1);			

				// after having loaded the position we check if the file is being played or not, then if it 
				// is not being played it is started, and begins to continue playing from the loaded position
				ui8_aux1=AY3891X_GetState();
				if (!((ui8_aux1==AY3891X_STATE_YM_READ_REGS_FROM_FILE)||(ui8_aux1==AY3891X_STATE_YM_LOAD_REGS_TO_AY))){
					// if the file is NOT being played we set the AY3891X main motor in PAUSE state
					AY3891X_SetState(AY3891X_STATE_YM_READ_REGS_FROM_FILE);
				}//if
						
				INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
			};//if
			
			// if Enter is pressed the state machin jumps to the next PLAYING screen
			if (INTERFACE_CheckSwButton(SWITCH3)==TRUE){
				INTERFACE_state.state=INTERFACE_STATE_FILE_PLAYING_SHOW_INFO;
				INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
			};//if
			
			if (INTERFACE_CheckSwButton(SWITCH4)==TRUE){
				
				AY3891X_Reset();// resets the audio IC
				AY3891X_ResetSong();// clears the song information
				
				// restores the FAT32 structure with the pervious directory
				if (FAT32_RestorePreviousDirElement()>=0){
					// jump to the FILE EXPLORER state
					INTERFACE_state.state=INTERFACE_STATE_FILE_EXPLORER;
					INTERFACE_state.refresh_path_entries=TRUE;// updates the entries in the LCD
					INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
				}else{
					// jump to the FILE ERROR state
					INTERFACE_state.state=INTERFACE_STATE_FILE_ERROR;
					INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
				};//if
			};//if
			break;

		case INTERFACE_STATE_FILE_ERROR:

			if (INTERFACE_CheckSwButton(SWITCH1|SWITCH2|SWITCH3|SWITCH4)==TRUE){
					INTERFACE_state.state=INTERFACE_STATE_INTRO;
					//INTERFACE_state.refresh_path_entries=TRUE;// the entries will be added to the char buffer
					INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
			};//if
			break;
			
		case INTERFACE_STATE_CARD_ERROR:
		
			if (INTERFACE_CheckSwButton(SWITCH1|SWITCH2|SWITCH3|SWITCH4)==TRUE){
					INTERFACE_state.state=INTERFACE_STATE_INTRO;
					INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
			};//if
			break;
	
		default:
			INTERFACE_state.state=INTERFACE_STATE_INTRO;
			INTERFACE_state.lcd_content_modified=TRUE;// raises the LCD flag
			break;
	};//switch

	INTERFACE_ResetSwButtonsState();

};//INTERFACE_Keyboard_Motor



/*	Description:
	Interface's keyboard state machine
	Recives:
	Returns:
	 By value:
	 By reference:
 */
void INTERFACE_Screen_Motor(){
	uint8_t ui8_aux1;
	int8_t i8_aux1;
	int8_t i8_aux2;
	uint16_t ui16_aux1;
	uint32_t ui32_aux1;
	uint32_t ui32_aux2;


	// if the flag has been raised we refresh the content of the interface
	if (INTERFACE_state.lcd_content_modified==TRUE){
		switch (INTERFACE_state.state){
			
			case INTERFACE_STATE_INTRO:
				// updates the LCD char buffer wit the INTRO SCREEN
				LCDG_SetString(0,7,"                  ");
				LCDG_SetString(0,6,"      -hard-      ");
				LCDG_SetString(0,5,"      plAYer      ");
				LCDG_SetString(0,4,"                  ");
				LCDG_SetString(0,3,"     Tolaemon     ");
				LCDG_SetString(0,2,"       2007       ");
				LCDG_SetString(0,1,"                  ");
				LCDG_SetString(0,0,"                  ");
				// prints the bottom line pushbutons signs
				LCDG_SetChar(11,0,193);//enter
				LCDG_SetChar(16,0,194);//exit
				LCDG_InvertRow(0xFF,0);// all lines unmarked
				LCDG_InvertRow(0,1);// only bottom line marked
				break;
			
			case INTERFACE_STATE_FILE_EXPLORER:
				// updates the LCD char buffer wit the FILE EXPLORER content

				if (INTERFACE_state.refresh_path_entries==TRUE){
					INTERFACE_state.refresh_path_entries=FALSE;
					
					// shows the clock symbol in the LCD while we are recalculating the content of the folder on the LCD
					LCDG_SetChar(9,4,200);
					LCDG_ForceLCDRefresh();
					
					LCDG_FillCharBuffer(' ');// clears the LCD					
					// prints each of the entries of the directory in each row of the LCD				
					i8_aux1=0;
					i8_aux2=0;
					while ((i8_aux1<NUMB_FILE_EXPL_ENTRIES)&&(i8_aux1<NUMB_FILE_EXPL_ENTRIES)&&(i8_aux2>=0)){
						
						// if there are still entries to print, they are printed		
						ui16_aux1=INTERFACE_state.file_expl_first+i8_aux1;
						i8_aux2=FAT32_GetDirElementN(ui16_aux1,short_entry,&ui8_aux1,&ui32_aux1,&ui32_aux2,long_entry);		
						if (i8_aux2>=0){
							
							// it the entry has been read without problems it's name is placed on the LCD buffer
							if (long_entry[0]=='\0'){
								// if the entry does not have long name entry we show it's short name entry
								LCDG_SetString(2,NUMB_FILE_EXPL_ENTRIES-i8_aux1,short_entry);
							}else{
								// if the entry has long name entry we show it
								LCDG_SetString(2,NUMB_FILE_EXPL_ENTRIES-i8_aux1,long_entry);
							};//if
										
							// then the element icon is printed depending on the type of the entry
							if (ui8_aux1==2){
								// if it is a file the file icon is printed
								if ((short_entry[9]=='Y')&&(short_entry[10]=='M')){
									// if it is a YM file the "YM icon" is printed
									LCDG_SetChar(0,NUMB_FILE_EXPL_ENTRIES-i8_aux1,183);
								}else{
									// if is NOT an YM file the "not YM file" icon is printed
									LCDG_SetChar(0,NUMB_FILE_EXPL_ENTRIES-i8_aux1,184);
								};//if
							}else{
								// if it is a directory, the directory icon is printed
								LCDG_SetChar(0,NUMB_FILE_EXPL_ENTRIES-i8_aux1,185);
							};//if
							LCDG_SetChar(1,NUMB_FILE_EXPL_ENTRIES-i8_aux1,'-');
							
						}else{
							
							// if any error has occured while reading the entry we go to FILE ERROR STATE
							INTERFACE_state.state=INTERFACE_STATE_FILE_ERROR;
							INTERFACE_state.lcd_content_modified=1;// raises the LCD flag
							
						};//if
						
						i8_aux1++;// process the next entry
						
					};//while

				};//if
		
				// marks up the currently selected entry
				LCDG_InvertRow(0xFF,0);// all lines unmarked
				i8_aux1=INTERFACE_state.file_expl_selected-INTERFACE_state.file_expl_first;
				LCDG_InvertRow(NUMB_FILE_EXPL_ENTRIES-i8_aux1,1);//marks up current selection
					
				// prints the bottom line pushbutons signs
				LCDG_SetChar(1,0,192);//down
				LCDG_SetChar(6,0,191);//up
				LCDG_SetChar(11,0,193);//enter
				LCDG_SetChar(16,0,194);//exit
				LCDG_InvertRow(0,1);// only bottom line marked
			
				break;
				
			case INTERFACE_STATE_FILE_LOADING:
				// updates the LCD char buffer with the LOADING message
				LCDG_SetString(0,7,"                  ");
				LCDG_SetString(0,6,"                  ");
				LCDG_SetString(0,5,"                  ");
				LCDG_SetString(0,4,"     Loading      ");
				LCDG_SetString(0,3,"                  ");
				LCDG_SetString(0,2,"                  ");
				LCDG_SetString(0,1,"                  ");
				LCDG_SetString(0,0,"                  ");
				LCDG_InvertRow(0xFF,0);// all lines unmarked
				break;	
			
			case INTERFACE_STATE_FILE_PLAYING_SHOW_INFO:
				LCDG_SetString(0,7,"    Song Info     ");

				// sets the file name into the LCD buffer
				LCDG_SetString(0,6,"                  ");
				LCDG_SetString(0,6,long_entry);
					
				// sets the song name into the LCD buffer					
				AY3891X_YM_GetSongName(file_info_string,MAX_AY_STRING);
				LCDG_SetString(0,5,"                  ");
				LCDG_SetString(0,5,file_info_string);
					
				// sets the author name into the LCD buffer					
				AY3891X_YM_GetSongAuthor(file_info_string,MAX_AY_STRING);
				LCDG_SetString(0,4,"                  ");
				LCDG_SetString(0,4,file_info_string);
					
				// sets the comments into the LCD buffer						
				AY3891X_YM_GetSongComments(file_info_string,MAX_AY_STRING);
				LCDG_SetString(0,3,"                  ");
				LCDG_SetString(0,3,file_info_string);
					
				// sets the clock freq into the LCD buffer
				AY3891X_YM_ClockType(&ui8_aux1);
				switch(ui8_aux1){
					case AMSTRAD_CPC_CLOCK:
						LCDG_SetString(0,2,"Amstrad CPC  clock");
						break;
					case SPECTRUM_ZX_CLOCK:
						LCDG_SetString(0,2,"Spectrum ZX  clock");
						break;
					case ATARI_ST_CLOCK:
						LCDG_SetString(0,2,"  Atari ST clock  ");
						break;
					default:
						break;
				};//switch

				// prints the bottom line pushbutons signs
				ui8_aux1=AY3891X_GetState();
				if ((ui8_aux1==AY3891X_STATE_YM_READ_REGS_FROM_FILE)||(ui8_aux1==AY3891X_STATE_YM_LOAD_REGS_TO_AY)){
					// if the file is being played we print the PAUSE icon
					LCDG_SetChar(1,0,199);//pause
				}else if (ui8_aux1==AY3891X_STATE_YM_PAUSED){
					// if the file is PAUSED we print the PLAY icon
					LCDG_SetChar(1,0,195);//play	
				}else if (ui8_aux1==AY3891X_STATE_YM_STOPED){
					// if the file is STOPPED we print the PLAY icon
					LCDG_SetChar(1,0,195);//play
				}//if
				LCDG_SetChar(6,0,198);//stop
				LCDG_SetChar(11,0,193);//prints enter
				LCDG_SetChar(16,0,194);//prints exit
				LCDG_InvertRow(0,1);// only bottom line marked
				
				break;
				
			case INTERFACE_STATE_FILE_PLAYING_SHOW_TIME:
				// updates the LCD char buffer with the information of played song
				LCDG_SetString(0,7,"     Playing      ");
				LCDG_SetString(0,6,"                  ");
				LCDG_SetString(0,5,"                  ");
				LCDG_SetString(0,4,"     :  /  :      ");
				LCDG_SetString(0,3,"                  ");
				LCDG_SetString(0,2,"                  ");
				LCDG_SetString(0,1,"                  ");
				LCDG_SetString(0,0,"                  ");		
				
				// prints son current position				
				AY3891X_YM_GetTotalAndCurrentFrame(&ui32_aux1,&ui32_aux2);
				ui16_aux1=(uint16_t)(ui32_aux2 / FRAMES_PER_MINUTE);
				LCD_SetUint(3,4,(uint32_t)ui16_aux1,2);
				ui16_aux1=(uint16_t) ( (ui32_aux2 % FRAMES_PER_MINUTE) / FRAMES_PER_SECOND) ;
				LCD_SetUint(6,4,(uint32_t)ui16_aux1,2);

				// prints song total time
				ui16_aux1=(uint16_t)(ui32_aux1 / FRAMES_PER_MINUTE);
				LCD_SetUint(9,4,(uint32_t)ui16_aux1,2);
				ui16_aux1=(uint16_t) ( (ui32_aux1 % FRAMES_PER_MINUTE) / FRAMES_PER_SECOND) ;
				LCD_SetUint(12,4,(uint32_t)ui16_aux1,2);

				// prints the bottom line pushbutons signs
				LCDG_SetChar(1,0,197);//down
				LCDG_SetChar(6,0,196);//up
				LCDG_SetChar(11,0,193);//enter
				LCDG_SetChar(16,0,194);//exit
				LCDG_InvertRow(0,1);// only bottom line marked
				
				break;
				
			case INTERFACE_STATE_FILE_PLAYING_CHANGE_SPPED:
				LCDG_SetString(0,7,"   Adjust speed   ");
				LCDG_SetString(0,6,"                  ");
				LCDG_SetString(0,5,"                  ");
				LCDG_SetString(0,4,"          ms/f    ");
				LCDG_SetString(0,3,"          f/s     ");
				LCDG_SetString(0,2,"                  ");
				LCDG_SetString(0,1,"                  ");
				LCDG_SetString(0,0,"                  ");

				// prints the tempo. To avoid working with decimal values and work with integers
				// this operation is done: (1000/((ui16_aux1*20)/3250)), starting from the inner 
				// brackets and continuing to the outer brackets
				AY3891X_GetFrameTimerCicles(&ui16_aux1);
				
				// the ms/f number
				ui32_aux1=(uint32_t)ui16_aux1*MILISECONDS_PER_FRAME;
				ui32_aux1=(uint32_t)ui32_aux1/FRAME_TIMER_CICLES;
				LCD_SetUint(5,4,(uint32_t)ui32_aux1,4);
				
				// the f/s number
				ui32_aux2=(uint32_t)1000/ui32_aux1; // conversion from 1000 second to seconds and from Period to Frecuency
				LCD_SetUint(6,3,(uint32_t)ui32_aux2,3);
		
				// prints the bottom line pushbutons signs
				LCDG_SetChar(1,0,192);//down
				LCDG_SetChar(6,0,191);//up
				LCDG_SetChar(11,0,193);//enter
				LCDG_SetChar(16,0,194);//exit
				LCDG_InvertRow(0,1);// only bottom line marked
				break;

			case INTERFACE_STATE_FILE_PLAYING_LOOPING:
				LCDG_SetString(0,7,"     Looping     ");
				LCDG_SetString(0,6,"                  ");
				LCDG_SetString(0,5,"                  ");
				LCDG_SetString(0,4,"        :         ");
				LCDG_SetString(0,3,"     :  /  :      ");
				LCDG_SetString(0,2,"                  ");
				LCDG_SetString(0,1,"                  ");
				LCDG_SetString(0,0,"                  ");	
				
				// prints song stored position				
				AY3891X_GetSongPositionFrame(1,&ui32_aux2);
				ui16_aux1=(uint16_t)(ui32_aux2 / FRAMES_PER_MINUTE);
				LCD_SetUint(6,4,(uint32_t)ui16_aux1,2);
				ui16_aux1=(uint16_t) ( (ui32_aux2 % FRAMES_PER_MINUTE) / FRAMES_PER_SECOND) ;
				LCD_SetUint(9,4,(uint32_t)ui16_aux1,2);
				
				// prints son current position				
				AY3891X_YM_GetTotalAndCurrentFrame(&ui32_aux1,&ui32_aux2);
				ui16_aux1=(uint16_t)(ui32_aux2 / FRAMES_PER_MINUTE);
				LCD_SetUint(3,3,(uint32_t)ui16_aux1,2);
				ui16_aux1=(uint16_t) ( (ui32_aux2 % FRAMES_PER_MINUTE) / FRAMES_PER_SECOND) ;
				LCD_SetUint(6,3,(uint32_t)ui16_aux1,2);
				
				// prints song total time
				ui16_aux1=(uint16_t)(ui32_aux1 / FRAMES_PER_MINUTE);
				LCD_SetUint(9,3,(uint32_t)ui16_aux1,2);
				ui16_aux1=(uint16_t) ( (ui32_aux1 % FRAMES_PER_MINUTE) / FRAMES_PER_SECOND) ;
				LCD_SetUint(12,3,(uint32_t)ui16_aux1,2);
				
				// prints the bottom line pushbutons signs
				LCDG_SetChar(1,0,'s');//S
				LCDG_SetChar(6,0,'l');//L
				LCDG_SetChar(11,0,193);//enter
				LCDG_SetChar(16,0,194);//exit
				LCDG_InvertRow(0,1);// only bottom line marked
				break;
				
			case INTERFACE_STATE_FILE_ERROR:
				LCDG_SetString(0,7,"                  ");
				LCDG_SetString(0,6,"                  ");
				LCDG_SetString(0,5,"                  ");
				LCDG_SetString(0,4,"   File  error!   ");
				LCDG_SetString(0,3,"                  ");
				LCDG_SetString(0,2,"                  ");
				LCDG_SetString(0,1,"                  ");
				LCDG_SetString(0,0,"                  ");
				LCDG_InvertRow(0xFF,0);// all lines unmarked
				// prints the bottom line pushbutons signs
				LCDG_SetChar(11,0,193);//enter
				LCDG_SetChar(16,0,194);//exit
				LCDG_InvertRow(0,1);// only bottom line marked
				break;	
				
			case INTERFACE_STATE_CARD_ERROR:
				LCDG_SetString(0,7,"                  ");
				LCDG_SetString(0,6,"                  ");
				LCDG_SetString(0,5,"                  ");
				LCDG_SetString(0,4,"Card is not ready!");
				LCDG_SetString(0,3,"                  ");
				LCDG_SetString(0,2,"                  ");
				LCDG_SetString(0,1,"                  ");
				LCDG_SetString(0,0,"                  ");
				LCDG_InvertRow(0xFF,0);// all lines unmarked
				// prints the bottom line pushbutons signs
				LCDG_SetChar(11,0,193);//enter
				LCDG_SetChar(16,0,194);//exit
				LCDG_InvertRow(0,1);// only bottom line marked
				break;	
				
			default:
				LCDG_SetString(0,7,"                  ");
				LCDG_SetString(0,6,"    SYSTEM ERROR  ");
				LCDG_SetString(0,5,"     UNDEFINED    ");
				LCDG_SetString(0,4,"       STATE      ");
				LCDG_SetString(0,3,"                  ");
				LCDG_SetString(0,2,"                  ");
				LCDG_SetString(0,1,"                  ");
				LCDG_SetString(0,0,"                  ");
				LCDG_InvertRow(0xFF,0);// all lines unmarked
				break;
				
		};//switch
		INTERFACE_state.lcd_content_modified=0;// disables the flag
	};//if

};//INTERFACE_Screen_Motor



/*	Description:
	State machine of the interface 
	Recives:
	Returns:
	 By value:
	 By reference:
 */
void INTERFACE_Motor(){

	INTERFACE_Keyboard_Motor();
	INTERFACE_Screen_Motor();

};//INTERFACE_Motor
